@@ -76,13 +76,13 @@ ffdfappend <- function(  x
                        ){
   
   fc <- if (is.ffdf(dat)){ 
-    sapply(physical(dat), function(i) is.factor.ff(i))
+    sapply(physical(dat)[names(dat)], function(i) is.factor.ff(i))
   } else {
     sapply(dat, function(i) is.factor(i) || is.character(i))    
   }
 
   if (!is.null(x)){
-    fc <- fc | sapply(physical(x), is.factor.ff)
+    fc <- fc | sapply(physical(x)[names(fc)], is.factor.ff)
   }
 
   if (any(fc) && !is.ffdf(dat)){
@@ -100,7 +100,7 @@ ffdfappend <- function(  x
    
    nrow(x) <- nff + n
   
-   for (i in which(fc)){
+   for (i in names(which(fc))){
      if(!is.factor.ff(x[[i]])){
        x[[i]] <- as.character.ff(x[[i]])
        #warning(sprintf("column %s of x is not a factor, column %s of dat is a factor, are you sure you want to ffdfappend", colnames(x)[i], colnames(dat)[i]))
@@ -128,5 +128,6 @@ ffdfappend <- function(  x
    i <- hi(nff+1, nff+n)
    x[i,] <- dat
    
-   x
+   # this makes sure that names(x) == physical(x), ff bug!!!
+   x[names(x)]
 }
