@@ -4,16 +4,29 @@
 #' @param laf laf object pointing to a csv or fwf file
 #' @param x optional, \code{\link{ffdf}} object where laf data should be appended to.
 #' @param nrows, number of rows per block, passed on to \code{next_block} 
+#' @param transFUN NULL or a function that is called on each data.frame chunk which is read in using \code{next_block}. 
+#' This can be used for filtering and data transformations.
 #' @param ... passed on to \code{next_block}
 #' @export
-laf_to_ffdf <- function(laf, x=NULL, nrows=1e5, ...){
+laf_to_ffdf <- function(laf, x=NULL, nrows=1e5, transFUN=NULL, ...){
   if (!require(LaF)){
     stop("This function needs the package 'LaF', which can be installed from CRAN")
   }
+  N <- 0
   begin(laf)
-  while(nrow(block <- next_block(laf, nrows=nrows, ...)))
+  Log$info("\n Adding laf to ffdf...")
+  while(nrow(block <- next_block(laf, nrows=nrows, ...))){
     #TODO test if adding columns separately is faster/or that allocating the ff vectors first is faster
-    x <- ffdfappend(x, block)
+    if(!is.null(transFUN)){
+      block <- transFUN(block)
+    }
+    if(nrow(block) > 0){
+      N <- N + nrow(block)
+      Log$info("\rRows added: ", N)
+      x <- ffdfappend(x, block)  
+    }
+    Log$info("\r")
+  }
   x
 }
 
@@ -43,4 +56,4 @@ laf_to_ffdf <- function(laf, x=NULL, nrows=1e5, ...){
 # for (n in c(1e3, 1e4, 1e5, 1e6)){
 #   cat(n, ":")
 #   cat(system.time(laf_to_ffdf(laf, nrows=n)))
-# }
\ No newline at end of file
+# }
